@@ -3,6 +3,7 @@
  *                                                                        *
  *   Copyright (C) 2001-2011, 2013-2019 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014 Mike Frysinger                                    *
+ *   Copyright (C) 2019 Brand Huntsman                                    *
  *   Copyright (C) 2014-2019 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -140,30 +141,54 @@ static bool opensyntax = FALSE;
 		 * or when a new syntax command is seen, this bool becomes FALSE. */
 static syntaxtype *live_syntax;
 		/* The syntax that is currently being parsed. */
+static bool seen_color_command = FALSE;
+		/* Whether a syntax definition contains any color commands. */
 static colortype *lastcolor = NULL;
 		/* The end of the color list for the current syntax. */
 #endif
 
-/* Report an error in an rcfile, printing it to stderr. */
-void rcfile_error(const char *msg, ...)
+static linestruct *errors_head = NULL;
+static linestruct *errors_tail = NULL;
+		/* Beginning and end of a list of errors in rcfiles, if any. */
+
+/* Send the gathered error messages (if any) to the terminal. */
+void display_rcfile_errors()
 {
+	for (linestruct *error = errors_head; error != NULL; error = error->next)
+		fprintf(stderr, "%s\n", error->data);
+}
+
+#define MAXSIZE  (PATH_MAX + 200)
+
+/* Store the given error message in a linked list, to be printed upon exit. */
+void jot_error(const char *msg, ...)
+{
+	linestruct *error = make_new_node(errors_tail);
+	char textbuf[MAXSIZE];
+	int length = 0;
 	va_list ap;
 
+	if (errors_head == NULL)
+		errors_head = error;
+	else
+		errors_tail->next = error;
+	errors_tail = error;
+
 	if (rcfile_with_errors == NULL)
 		rcfile_with_errors = strdup(nanorc);
 
 	if (lineno > 0)
-		fprintf(stderr, _("Error in %s on line %zu: "), nanorc, lineno);
+		length = snprintf(textbuf, MAXSIZE, _("Error in %s on line %zu: "),
+											nanorc, lineno);
 
 	va_start(ap, msg);
-	vfprintf(stderr, _(msg), ap);
+	length += vsnprintf(textbuf + length, MAXSIZE - length, _(msg), ap);
 	va_end(ap);
 
-	fprintf(stderr, "\n");
+	error->data = nmalloc(length + 1);
+	sprintf(error->data, "%s", textbuf);
 }
-#endif /* ENABLE_NANORC */
 
-#if defined(ENABLE_NANORC) || defined(ENABLE_HISTORIES)
 /* Parse the next word from the string, null-terminate it, and return
  * a pointer to the first character after the null terminator.  The
  * returned pointer will point to '\0' if we hit the end of the line. */
@@ -183,9 +208,7 @@ char *parse_next_word(char *ptr)
 
 	return ptr;
 }
-#endif /* ENABLE_NANORC || ENABLE_HISTORIES */
 
-#ifdef ENABLE_NANORC
 /* Parse an argument, with optional quotes, after a keyword that takes
  * one.  If the next word starts with a ", we say that it ends with the
  * last " of the line.  Otherwise, we interpret it as usual, so that the
@@ -204,7 +227,7 @@ char *parse_argument(char *ptr)
 	}
 
 	if (last_quote == NULL) {
-		rcfile_error(N_("Argument '%s' has an unterminated \""), ptr_save);
+		jot_error(N_("Argument '%s' has an unterminated \""), ptr_save);
 		return NULL;
 	}
 
@@ -229,7 +252,7 @@ char *parse_next_regex(char *ptr)
 		ptr++;
 
 	if (*ptr == '\0') {
-		rcfile_error(N_("Regex strings must begin and end with a \" character"));
+		jot_error(N_("Regex strings must begin and end with a \" character"));
 		return NULL;
 	}
 
@@ -242,38 +265,41 @@ char *parse_next_regex(char *ptr)
 	return ptr;
 }
 
-/* Compile the regular expression regex to see if it's valid.  Return
- * TRUE if it is, and FALSE otherwise. */
-bool nregcomp(const char *regex, int compile_flags)
+/* Compile the given regular expression and store the result in packed (when
+ * this pointer is not NULL).  Return TRUE when the expression is valid. */
+bool compile(const char *expression, int rex_flags, regex_t **packed)
 {
-	regex_t preg;
-	int rc = regcomp(&preg, regex, compile_flags);
+	regex_t *compiled = nmalloc(sizeof(regex_t));
+	int outcome = regcomp(compiled, expression, rex_flags);
 
-	if (rc != 0) {
-		size_t len = regerror(rc, &preg, NULL, 0);
-		char *str = charalloc(len);
+	if (outcome != 0) {
+		size_t length = regerror(outcome, compiled, NULL, 0);
+		char *message = charalloc(length);
 
-		regerror(rc, &preg, str, len);
-		rcfile_error(N_("Bad regex \"%s\": %s"), regex, str);
-		free(str);
+		regerror(outcome, compiled, message, length);
+		jot_error(N_("Bad regex \"%s\": %s"), expression, message);
+		free(message);
 	}
 
-	regfree(&preg);
-	return (rc == 0);
+	if (packed == NULL || outcome != 0) {
+		regfree(compiled);
+		free(compiled);
+	} else
+		*packed = compiled;
+
+	return (outcome == 0);
 }
 
 /* Parse the next syntax name and its possible extension regexes from the
  * line at ptr, and add it to the global linked list of color syntaxes. */
-void parse_syntax(char *ptr)
+void begin_new_syntax(char *ptr)
 {
 	char *nameptr = ptr;
 
-	opensyntax = FALSE;
-
 	/* Check that the syntax name is not empty. */
 	if (*ptr == '\0' || (*ptr == '"' &&
 						(*(ptr + 1) == '\0' || *(ptr + 1) == '"'))) {
-		rcfile_error(N_("Missing syntax name"));
+		jot_error(N_("Missing syntax name"));
 		return;
 	}
 
@@ -281,7 +307,7 @@ void parse_syntax(char *ptr)
 
 	/* Check that quotes around the name are either paired or absent. */
 	if ((*nameptr == '\x22') ^ (nameptr[strlen(nameptr) - 1] == '\x22')) {
-		rcfile_error(N_("Unpaired quote in syntax name"));
+		jot_error(N_("Unpaired quote in syntax name"));
 		return;
 	}
 
@@ -293,13 +319,16 @@ void parse_syntax(char *ptr)
 
 	/* Redefining the "none" syntax is not allowed. */
 	if (strcmp(nameptr, "none") == 0) {
-		rcfile_error(N_("The \"none\" syntax is reserved"));
+		jot_error(N_("The \"none\" syntax is reserved"));
 		return;
 	}
 
 	/* Initialize a new syntax struct. */
 	live_syntax = (syntaxtype *)nmalloc(sizeof(syntaxtype));
 	live_syntax->name = mallocstrcpy(NULL, nameptr);
+	live_syntax->filename = strdup(nanorc);
+	live_syntax->lineno = lineno;
+	live_syntax->augmentations = NULL;
 	live_syntax->extensions = NULL;
 	live_syntax->headers = NULL;
 	live_syntax->magics = NULL;
@@ -308,7 +337,6 @@ void parse_syntax(char *ptr)
 	live_syntax->comment = mallocstrcpy(NULL, GENERAL_COMMENT_CHARACTER);
 #endif
 	live_syntax->color = NULL;
-	lastcolor = NULL;
 	live_syntax->nmultis = 0;
 
 	/* Hook the new syntax in at the top of the list. */
@@ -316,10 +344,11 @@ void parse_syntax(char *ptr)
 	syntaxes = live_syntax;
 
 	opensyntax = TRUE;
+	seen_color_command = FALSE;
 
 	/* The default syntax should have no associated extensions. */
 	if (strcmp(live_syntax->name, "default") == 0 && *ptr != '\0') {
-		rcfile_error(N_("The \"default\" syntax does not accept extensions"));
+		jot_error(N_("The \"default\" syntax does not accept extensions"));
 		return;
 	}
 
@@ -329,6 +358,17 @@ void parse_syntax(char *ptr)
 }
 #endif /* ENABLE_COLOR */
 
+/* Verify that a syntax definition contains at least one color command. */
+void check_for_nonempty_syntax(void)
+{
+#ifdef ENABLE_COLOR
+	if (opensyntax && !seen_color_command)
+		jot_error(N_("Syntax \"%s\" has no color commands"), live_syntax->name);
+
+	opensyntax = FALSE;
+#endif
+}
+
 /* Return TRUE when the given function is present in almost all menus. */
 bool is_universal(void (*func)(void))
 {
@@ -338,7 +378,7 @@ bool is_universal(void (*func)(void))
 		func == do_prev_word_void || func == do_next_word_void ||
 #endif
 		func == do_delete || func == do_backspace ||
-		func == do_cut_text_void || func == do_uncut_text ||
+		func == cut_text || func == paste_text ||
 		func == do_tab || func == do_enter || func == do_verbatim_input);
 }
 
@@ -350,8 +390,10 @@ void parse_binding(char *ptr, bool dobind)
 	int menu, mask = 0;
 	funcstruct *f;
 
+	check_for_nonempty_syntax();
+
 	if (*ptr == '\0') {
-		rcfile_error(N_("Missing key name"));
+		jot_error(N_("Missing key name"));
 		return;
 	}
 
@@ -360,7 +402,7 @@ void parse_binding(char *ptr, bool dobind)
 	keycopy = mallocstrcpy(NULL, keyptr);
 
 	if (strlen(keycopy) < 2) {
-		rcfile_error(N_("Key name is too short"));
+		jot_error(N_("Key name is too short"));
 		goto free_things;
 	}
 
@@ -371,7 +413,7 @@ void parse_binding(char *ptr, bool dobind)
 		if (strlen(keycopy) > 2)
 			keycopy[2] = toupper((unsigned char)keycopy[2]);
 		else {
-			rcfile_error(N_("Key name is too short"));
+			jot_error(N_("Key name is too short"));
 			goto free_things;
 		}
 	}
@@ -381,10 +423,10 @@ void parse_binding(char *ptr, bool dobind)
 	if (!strcasecmp(keycopy, "Ins") || !strcasecmp(keycopy, "Del"))
 		keycopy[1] = tolower((unsigned char)keycopy[1]);
 	else if (keycopy[0] != '^' && keycopy[0] != 'M' && keycopy[0] != 'F') {
-		rcfile_error(N_("Key name must begin with \"^\", \"M\", or \"F\""));
+		jot_error(N_("Key name must begin with \"^\", \"M\", or \"F\""));
 		goto free_things;
 	} else if (keycode_from_string(keycopy) < 0) {
-		rcfile_error(N_("Key name %s is invalid"), keycopy);
+		jot_error(N_("Key name %s is invalid"), keycopy);
 		goto free_things;
 	}
 
@@ -393,7 +435,7 @@ void parse_binding(char *ptr, bool dobind)
 		ptr = parse_argument(ptr);
 
 		if (funcptr[0] == '\0') {
-			rcfile_error(N_("Must specify a function to bind the key to"));
+			jot_error(N_("Must specify a function to bind the key to"));
 			goto free_things;
 		} else if (ptr == NULL)
 			goto free_things;
@@ -404,7 +446,8 @@ void parse_binding(char *ptr, bool dobind)
 
 	if (menuptr[0] == '\0') {
 		/* TRANSLATORS: Do not translate the word "all". */
-		rcfile_error(N_("Must specify a menu (or \"all\") in which to bind/unbind the key"));
+		jot_error(N_("Must specify a menu (or \"all\") "
+						"in which to bind/unbind the key"));
 		goto free_things;
 	}
 
@@ -422,14 +465,14 @@ void parse_binding(char *ptr, bool dobind)
 			newsc = strtosc(funcptr);
 
 		if (newsc == NULL) {
-			rcfile_error(N_("Cannot map name \"%s\" to a function"), funcptr);
+			jot_error(N_("Cannot map name \"%s\" to a function"), funcptr);
 			goto free_things;
 		}
 	}
 
 	menu = name_to_menu(menuptr);
 	if (menu < 1) {
-		rcfile_error(N_("Cannot map name \"%s\" to a menu"), menuptr);
+		jot_error(N_("Cannot map name \"%s\" to a menu"), menuptr);
 		goto free_things;
 	}
 
@@ -462,7 +505,7 @@ void parse_binding(char *ptr, bool dobind)
 
 	if (!menu) {
 		if (!ISSET(RESTRICTED) && !ISSET(VIEW_MODE))
-			rcfile_error(N_("Function '%s' does not exist in menu '%s'"),
+			jot_error(N_("Function '%s' does not exist in menu '%s'"),
 								funcptr, menuptr);
 		goto free_things;
 	}
@@ -473,7 +516,7 @@ void parse_binding(char *ptr, bool dobind)
 	/* Disallow rebinding ^[ and frequent escape-sequence starter "Esc [". */
 	if ((!newsc->meta && newsc->keycode == ESC_CODE) ||
 				(newsc->meta && newsc->keycode == '[')) {
-		rcfile_error(N_("Keystroke %s may not be rebound"), keycopy);
+		jot_error(N_("Keystroke %s may not be rebound"), keycopy);
   free_things:
 		free(keycopy);
 		free(newsc);
@@ -506,8 +549,8 @@ bool is_good_file(char *file)
 	/* If the thing exists, it may be neither a directory nor a device. */
 	if (stat(file, &rcinfo) != -1 && (S_ISDIR(rcinfo.st_mode) ||
 				S_ISCHR(rcinfo.st_mode) || S_ISBLK(rcinfo.st_mode))) {
-		rcfile_error(S_ISDIR(rcinfo.st_mode) ? _("\"%s\" is a directory") :
-										_("\"%s\" is a device file"), file);
+		jot_error(S_ISDIR(rcinfo.st_mode) ? N_("\"%s\" is a directory") :
+										N_("\"%s\" is a device file"), file);
 		return FALSE;
 	} else
 		return TRUE;
@@ -515,8 +558,9 @@ bool is_good_file(char *file)
 
 #ifdef ENABLE_COLOR
 /* Read and parse one included syntax file. */
-static void parse_one_include(char *file)
+void parse_one_include(char *file, syntaxtype *syntax)
 {
+	augmentstruct *extra;
 	FILE *rcstream;
 
 	/* Don't open directories, character files, or block files. */
@@ -527,7 +571,7 @@ static void parse_one_include(char *file)
 	rcstream = fopen(file, "rb");
 
 	if (rcstream == NULL) {
-		rcfile_error(_("Error reading %s: %s"), file, strerror(errno));
+		jot_error(N_("Error reading %s: %s"), file, strerror(errno));
 		return;
 	}
 
@@ -536,7 +580,36 @@ static void parse_one_include(char *file)
 	nanorc = file;
 	lineno = 0;
 
-	parse_rcfile(rcstream, TRUE);
+	/* If this is the first pass, parse only the prologue. */
+	if (syntax == NULL) {
+		parse_rcfile(rcstream, TRUE, TRUE);
+		return;
+	}
+
+	live_syntax = syntax;
+	lastcolor = NULL;
+
+	/* Fully parse the given syntax (as it is about to be used). */
+	parse_rcfile(rcstream, TRUE, FALSE);
+
+	extra = syntax->augmentations;
+
+	/* Apply any stored extendsyntax commands. */
+	while (extra != NULL) {
+		char *keyword = extra->data;
+		char *therest = parse_next_word(extra->data);
+
+		nanorc = extra->filename;
+		lineno = extra->lineno;
+
+		if (!parse_syntax_commands(keyword, therest))
+			jot_error(N_("Command \"%s\" not understood"), keyword);
+
+		extra = extra->next;
+	}
+
+	free(syntax->filename);
+	syntax->filename = NULL;
 }
 
 /* Expand globs in the passed name, and parse the resultant files. */
@@ -561,9 +634,9 @@ void parse_includes(char *ptr)
 	 * report an error if it's something other than zero matches. */
 	if (result == 0) {
 		for (size_t i = 0; i < files.gl_pathc; ++i)
-			parse_one_include(files.gl_pathv[i]);
+			parse_one_include(files.gl_pathv[i], NULL);
 	} else if (result != GLOB_NOMATCH)
-		rcfile_error(_("Error expanding %s: %s"), pattern, strerror(errno));
+		jot_error(N_("Error expanding %s: %s"), pattern, strerror(errno));
 
 	globfree(&files);
 	free(expanded);
@@ -603,7 +676,7 @@ short color_to_short(const char *colorname, bool *bright)
 	else if (strcasecmp(colorname, "normal") == 0)
 		return USE_THE_DEFAULT;
 
-	rcfile_error(N_("Color \"%s\" not understood"), colorname);
+	jot_error(N_("Color \"%s\" not understood"), colorname);
 	return BAD_COLOR;
 }
 
@@ -619,7 +692,7 @@ bool parse_color_names(char *combostr, short *fg, short *bg, int *attributes)
 	if (comma != NULL) {
 		*bg = color_to_short(comma + 1, &bright);
 		if (bright) {
-			rcfile_error(N_("A background color cannot be bright"));
+			jot_error(N_("A background color cannot be bright"));
 			return FALSE;
 		}
 		if (*bg == BAD_COLOR)
@@ -650,14 +723,8 @@ void parse_colors(char *ptr, int rex_flags)
 	int attributes;
 	char *item;
 
-	if (!opensyntax) {
-		rcfile_error(N_("A '%s' command requires a preceding 'syntax' command"),
-						"color");
-		return;
-	}
-
 	if (*ptr == '\0') {
-		rcfile_error(N_("Missing color name"));
+		jot_error(N_("Missing color name"));
 		return;
 	}
 
@@ -667,7 +734,7 @@ void parse_colors(char *ptr, int rex_flags)
 		return;
 
 	if (*ptr == '\0') {
-		rcfile_error(N_("Missing regex string after '%s' command"), "color");
+		jot_error(N_("Missing regex string after '%s' command"), "color");
 		return;
 	}
 
@@ -687,7 +754,7 @@ void parse_colors(char *ptr, int rex_flags)
 		}
 
 		if (*ptr != '"') {
-			rcfile_error(N_("Regex strings must begin and end with a \" character"));
+			jot_error(N_("Regex strings must begin and end with a \" character"));
 			ptr = parse_next_regex(ptr);
 			continue;
 		}
@@ -698,27 +765,21 @@ void parse_colors(char *ptr, int rex_flags)
 			break;
 
 		if (*item == '\0') {
-			rcfile_error(N_("Empty regex string"));
+			jot_error(N_("Empty regex string"));
 			goodstart = FALSE;
-		} else
-			goodstart = nregcomp(item, rex_flags);
-
-		/* If the starting regex is valid, initialize a new color struct,
-		 * and hook it in at the tail of the linked list. */
-		if (goodstart) {
+		} else {
 			newcolor = (colortype *)nmalloc(sizeof(colortype));
+			goodstart = compile(item, rex_flags, &newcolor->start);
+		}
 
+		/* If the start regex is valid, fill in the rest of the data, and
+		 * hook the new color struct in at the tail of the linked list. */
+		if (goodstart) {
 			newcolor->fg = fg;
 			newcolor->bg = bg;
 			newcolor->attributes = attributes;
-			newcolor->rex_flags = rex_flags;
 
-			newcolor->start_regex = mallocstrcpy(NULL, item);
-			newcolor->start = NULL;
-
-			newcolor->end_regex = NULL;
 			newcolor->end = NULL;
-
 			newcolor->next = NULL;
 
 			if (lastcolor == NULL)
@@ -727,19 +788,20 @@ void parse_colors(char *ptr, int rex_flags)
 				lastcolor->next = newcolor;
 
 			lastcolor = newcolor;
-		}
+		} else
+			free(newcolor);
 
 		if (!expectend)
 			continue;
 
 		if (ptr == NULL || strncasecmp(ptr, "end=", 4) != 0) {
-			rcfile_error(N_("\"start=\" requires a corresponding \"end=\""));
+			jot_error(N_("\"start=\" requires a corresponding \"end=\""));
 			return;
 		}
 
 		ptr += 4;
 		if (*ptr != '"') {
-			rcfile_error(N_("Regex strings must begin and end with a \" character"));
+			jot_error(N_("Regex strings must begin and end with a \" character"));
 			continue;
 		}
 
@@ -749,18 +811,16 @@ void parse_colors(char *ptr, int rex_flags)
 			break;
 
 		if (*item == '\0') {
-			rcfile_error(N_("Empty regex string"));
+			jot_error(N_("Empty regex string"));
 			continue;
 		}
 
-		/* If the start regex was invalid, skip past the end regex
-		 * to stay in sync. */
+		/* If the start regex was invalid, the end regex cannot be saved. */
 		if (!goodstart)
 			continue;
 
-		/* If it's valid, save the ending regex string. */
-		if (nregcomp(item, rex_flags))
-			newcolor->end_regex = mallocstrcpy(NULL, item);
+		/* Save the compiled ending regex (when it's valid). */
+		compile(item, rex_flags, &newcolor->end);
 
 		/* Lame way to skip another static counter. */
 		newcolor->id = live_syntax->nmultis;
@@ -790,20 +850,18 @@ void grab_and_store(const char *kind, char *ptr, regexlisttype **storage)
 	regexlisttype *lastthing;
 
 	if (!opensyntax) {
-		rcfile_error(
-				N_("A '%s' command requires a preceding 'syntax' command"), kind);
+		jot_error(N_("A '%s' command requires a preceding 'syntax' command"), kind);
 		return;
 	}
 
 	/* The default syntax doesn't take any file matching stuff. */
 	if (strcmp(live_syntax->name, "default") == 0 && *ptr != '\0') {
-		rcfile_error(
-				N_("The \"default\" syntax does not accept '%s' regexes"), kind);
+		jot_error(N_("The \"default\" syntax does not accept '%s' regexes"), kind);
 		return;
 	}
 
 	if (*ptr == '\0') {
-		rcfile_error(N_("Missing regex string after '%s' command"), kind);
+		jot_error(N_("Missing regex string after '%s' command"), kind);
 		return;
 	}
 
@@ -819,7 +877,7 @@ void grab_and_store(const char *kind, char *ptr, regexlisttype **storage)
 		regexlisttype *newthing;
 
 		if (*ptr != '"') {
-			rcfile_error(N_("Regex strings must begin and end with a \" character"));
+			jot_error(N_("Regex strings must begin and end with a \" character"));
 			return;
 		}
 
@@ -829,7 +887,7 @@ void grab_and_store(const char *kind, char *ptr, regexlisttype **storage)
 			return;
 
 		/* If the regex string is malformed, skip it. */
-		if (!nregcomp(regexstring, NANO_REG_EXTENDED | REG_NOSUB))
+		if (!compile(regexstring, NANO_REG_EXTENDED | REG_NOSUB, NULL))
 			continue;
 
 		/* Copy the regex into a struct, and hook this in at the end. */
@@ -849,14 +907,8 @@ void grab_and_store(const char *kind, char *ptr, regexlisttype **storage)
 /* Gather and store the string after a comment/linter command. */
 void pick_up_name(const char *kind, char *ptr, char **storage)
 {
-	if (!opensyntax) {
-		rcfile_error(
-				N_("A '%s' command requires a preceding 'syntax' command"), kind);
-		return;
-	}
-
 	if (*ptr == '\0') {
-		rcfile_error(N_("Missing argument after '%s'"), kind);
+		jot_error(N_("Missing argument after '%s'"), kind);
 		return;
 	}
 
@@ -866,7 +918,7 @@ void pick_up_name(const char *kind, char *ptr, char **storage)
 
 		while (*look != '"') {
 			if (--look == ptr) {
-				rcfile_error(N_("Argument of '%s' lacks closing \""), kind);
+				jot_error(N_("Argument of '%s' lacks closing \""), kind);
 				return;
 			}
 		}
@@ -877,6 +929,25 @@ void pick_up_name(const char *kind, char *ptr, char **storage)
 
 	*storage = mallocstrcpy(*storage, ptr);
 }
+
+/* Handle the four syntax-only commands. */
+bool parse_syntax_commands(char *keyword, char *ptr)
+{
+	if (strcasecmp(keyword, "color") == 0)
+		parse_colors(ptr, NANO_REG_EXTENDED);
+	else if (strcasecmp(keyword, "icolor") == 0)
+		parse_colors(ptr, NANO_REG_EXTENDED | REG_ICASE);
+	else if (strcasecmp(keyword, "comment") == 0) {
+#ifdef ENABLE_COMMENT
+		pick_up_name("comment", ptr, &live_syntax->comment);
+#endif
+	} else if (strcasecmp(keyword, "linter") == 0)
+		pick_up_name("linter", ptr, &live_syntax->linter);
+	else
+		return FALSE;
+
+	return TRUE;
+}
 #endif /* ENABLE_COLOR */
 
 /* Verify that the user has not unmapped every shortcut for a
@@ -908,25 +979,31 @@ static void check_vitals_mapped(void)
 }
 
 /* Parse the rcfile, once it has been opened successfully at rcstream,
- * and close it afterwards.  If syntax_only is TRUE, allow the file to
+ * and close it afterwards.  If just_syntax is TRUE, allow the file to
  * to contain only color syntax commands. */
-void parse_rcfile(FILE *rcstream, bool syntax_only)
+void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 {
-	char *buf = NULL;
+	char *buffer = NULL;
 	ssize_t len;
-	size_t n = 0;
+	size_t size = 0;
 
-	while ((len = getline(&buf, &n, rcstream)) > 0) {
+	while ((len = getline(&buffer, &size, rcstream)) > 0) {
 		char *ptr, *keyword, *option;
 		int set = 0;
 		size_t i;
 
-		/* Ignore the newline. */
-		if (buf[len - 1] == '\n')
-			buf[len - 1] = '\0';
-
 		lineno++;
-		ptr = buf;
+
+#ifdef ENABLE_COLOR
+		/* If doing a full parse, skip to after the 'syntax' command. */
+		if (just_syntax && !intros_only && lineno <= live_syntax->lineno)
+			continue;
+#endif
+		/* Strip the terminating newline, if any. */
+		if (buffer[len - 1] == '\n')
+			buffer[len - 1] = '\0';
+
+		ptr = buffer;
 		while (isblank((unsigned char)*ptr))
 			ptr++;
 
@@ -940,9 +1017,12 @@ void parse_rcfile(FILE *rcstream, bool syntax_only)
 
 #ifdef ENABLE_COLOR
 		/* Handle extending first... */
-		if (strcasecmp(keyword, "extendsyntax") == 0 && !syntax_only) {
-			syntaxtype *sint;
+		if (!just_syntax && strcasecmp(keyword, "extendsyntax") == 0) {
+			augmentstruct *newitem, *extra;
 			char *syntaxname = ptr;
+			syntaxtype *sint;
+
+			check_for_nonempty_syntax();
 
 			ptr = parse_next_word(ptr);
 
@@ -951,60 +1031,67 @@ void parse_rcfile(FILE *rcstream, bool syntax_only)
 					break;
 
 			if (sint == NULL) {
-				rcfile_error(N_("Could not find syntax \"%s\" to extend"),
-								syntaxname);
-				opensyntax = FALSE;
+				jot_error(N_("Could not find syntax \"%s\" to extend"), syntaxname);
 				continue;
 			}
 
-			live_syntax = sint;
-			opensyntax = TRUE;
+			newitem = nmalloc(sizeof(augmentstruct));;
 
-			/* Refind the tail of the color list for this syntax. */
-			lastcolor = sint->color;
-			if (lastcolor != NULL)
-				while (lastcolor->next != NULL)
-					lastcolor = lastcolor->next;
+			/* Store the content of an 'extendsyntax', for later parsing. */
+			newitem->filename = strdup(nanorc);
+			newitem->lineno = lineno;
+			newitem->data = strdup(ptr);
+			newitem->next = NULL;
 
-			keyword = ptr;
-			ptr = parse_next_word(ptr);
+			if (sint->augmentations != NULL) {
+				extra = sint->augmentations;
+				while (extra->next != NULL)
+					extra = extra->next;
+				extra->next = newitem;
+			} else
+				sint->augmentations = newitem;
+
+			continue;
 		}
 
 		/* Try to parse the keyword. */
 		if (strcasecmp(keyword, "syntax") == 0) {
-			if (opensyntax && lastcolor == NULL)
-				rcfile_error(N_("Syntax \"%s\" has no color commands"),
-								live_syntax->name);
-			parse_syntax(ptr);
-		}
-		else if (strcasecmp(keyword, "header") == 0)
-			grab_and_store("header", ptr, &live_syntax->headers);
-		else if (strcasecmp(keyword, "magic") == 0)
+			if (intros_only) {
+				check_for_nonempty_syntax();
+				begin_new_syntax(ptr);
+			} else
+				break;
+		} else if (strcasecmp(keyword, "header") == 0) {
+			if (intros_only)
+				grab_and_store("header", ptr, &live_syntax->headers);
+		} else if (strcasecmp(keyword, "magic") == 0) {
 #ifdef HAVE_LIBMAGIC
-			grab_and_store("magic", ptr, &live_syntax->magics);
-#else
-			;
+			if (intros_only)
+				grab_and_store("magic", ptr, &live_syntax->magics);
 #endif
-		else if (strcasecmp(keyword, "comment") == 0)
-#ifdef ENABLE_COMMENT
-			pick_up_name("comment", ptr, &live_syntax->comment);
-#else
-			;
-#endif
-		else if (strcasecmp(keyword, "color") == 0)
-			parse_colors(ptr, NANO_REG_EXTENDED);
-		else if (strcasecmp(keyword, "icolor") == 0)
-			parse_colors(ptr, NANO_REG_EXTENDED | REG_ICASE);
-		else if (strcasecmp(keyword, "linter") == 0)
-			pick_up_name("linter", ptr, &live_syntax->linter);
-		else if (syntax_only && (strcasecmp(keyword, "set") == 0 ||
+		} else if (just_syntax && (strcasecmp(keyword, "set") == 0 ||
 								strcasecmp(keyword, "unset") == 0 ||
 								strcasecmp(keyword, "bind") == 0 ||
 								strcasecmp(keyword, "unbind") == 0 ||
 								strcasecmp(keyword, "include") == 0 ||
-								strcasecmp(keyword, "extendsyntax") == 0))
-			rcfile_error(N_("Command \"%s\" not allowed in included file"),
-										keyword);
+								strcasecmp(keyword, "extendsyntax") == 0)) {
+			if (intros_only)
+				jot_error(N_("Command \"%s\" not allowed in included file"),
+									keyword);
+			else
+				break;
+		} else if (intros_only && (strcasecmp(keyword, "color") == 0 ||
+								strcasecmp(keyword, "icolor") == 0 ||
+								strcasecmp(keyword, "comment") == 0 ||
+								strcasecmp(keyword, "linter") == 0)) {
+			if (!opensyntax)
+				jot_error(N_("A '%s' command requires a preceding "
+									"'syntax' command"), keyword);
+			if (strcasestr("icolor", keyword))
+				seen_color_command = TRUE;
+			continue;
+		} else if (parse_syntax_commands(keyword, ptr))
+			;
 		else if (strcasecmp(keyword, "include") == 0)
 			parse_includes(ptr);
 		else
@@ -1017,56 +1104,50 @@ void parse_rcfile(FILE *rcstream, bool syntax_only)
 			parse_binding(ptr, TRUE);
 		else if (strcasecmp(keyword, "unbind") == 0)
 			parse_binding(ptr, FALSE);
-		else
-			rcfile_error(N_("Command \"%s\" not understood"), keyword);
-
-#ifdef ENABLE_COLOR
-		/* If a syntax was extended, it stops at the end of the command. */
-		if (live_syntax != syntaxes)
-			opensyntax = FALSE;
-#endif
+		else if (intros_only)
+			jot_error(N_("Command \"%s\" not understood"), keyword);
 
 		if (set == 0)
 			continue;
 
+		check_for_nonempty_syntax();
+
 		if (*ptr == '\0') {
-			rcfile_error(N_("Missing option"));
+			jot_error(N_("Missing option"));
 			continue;
 		}
 
 		option = ptr;
 		ptr = parse_next_word(ptr);
 
-		/* Find the just read name among the existing options. */
+		/* Find the just parsed option name among the existing names. */
 		for (i = 0; rcopts[i].name != NULL; i++) {
 			if (strcasecmp(option, rcopts[i].name) == 0)
 				break;
 		}
 
 		if (rcopts[i].name == NULL) {
-			rcfile_error(N_("Unknown option \"%s\""), option);
+			jot_error(N_("Unknown option \"%s\""), option);
 			continue;
 		}
 
-		/* First handle unsetting. */
-		if (set == -1) {
-			if (rcopts[i].flag != 0)
-				UNSET(rcopts[i].flag);
+		/* If the option has a flag, set it or unset it, as requested. */
+		if (rcopts[i].flag) {
+			if (set == 1)
+				SET(rcopts[i].flag);
 			else
-				rcfile_error(N_("Cannot unset option \"%s\""), rcopts[i].name);
+				UNSET(rcopts[i].flag);
 			continue;
 		}
 
-		/* If the option has a flag, it doesn't take an argument. */
-		if (rcopts[i].flag != 0) {
-			SET(rcopts[i].flag);
+		/* An option that takes an argument cannot be unset. */
+		if (set == -1) {
+			jot_error(N_("Cannot unset option \"%s\""), rcopts[i].name);
 			continue;
 		}
 
-		/* The option doesn't have a flag, so it takes an argument. */
 		if (*ptr == '\0') {
-			rcfile_error(N_("Option \"%s\" requires an argument"),
-								rcopts[i].name);
+			jot_error(N_("Option \"%s\" requires an argument"), rcopts[i].name);
 			continue;
 		}
 
@@ -1080,7 +1161,7 @@ void parse_rcfile(FILE *rcstream, bool syntax_only)
 #ifdef ENABLE_UTF8
 		/* When in a UTF-8 locale, ignore arguments with invalid sequences. */
 		if (using_utf8() && mbstowcs(NULL, option, 0) == (size_t)-1) {
-			rcfile_error(N_("Argument is not a valid multibyte string"));
+			jot_error(N_("Argument is not a valid multibyte string"));
 			continue;
 		}
 #endif
@@ -1111,8 +1192,7 @@ void parse_rcfile(FILE *rcstream, bool syntax_only)
 #ifdef ENABLED_WRAPORJUSTIFY
 		if (strcasecmp(rcopts[i].name, "fill") == 0) {
 			if (!parse_num(option, &fill)) {
-				rcfile_error(N_("Requested fill size \"%s\" is invalid"),
-								option);
+				jot_error(N_("Requested fill size \"%s\" is invalid"), option);
 				fill = -COLUMNS_FROM_EOL;
 			}
 			free(option);
@@ -1121,40 +1201,40 @@ void parse_rcfile(FILE *rcstream, bool syntax_only)
 #ifndef NANO_TINY
 		if (strcasecmp(rcopts[i].name, "guidestripe") == 0) {
 			if (!parse_num(option, &stripe_column) || stripe_column <= 0) {
-				rcfile_error(N_("Guide column \"%s\" is invalid"), option);
+				jot_error(N_("Guide column \"%s\" is invalid"), option);
 				stripe_column = 0;
 			}
 			free(option);
 		} else if (strcasecmp(rcopts[i].name, "matchbrackets") == 0) {
 			if (has_blank_char(option)) {
-				rcfile_error(N_("Non-blank characters required"));
+				jot_error(N_("Non-blank characters required"));
 				free(option);
 			} else if (mbstrlen(option) % 2 != 0) {
-				rcfile_error(N_("Even number of characters required"));
+				jot_error(N_("Even number of characters required"));
 				free(option);
 			} else
 				matchbrackets = option;
 		} else if (strcasecmp(rcopts[i].name, "whitespace") == 0) {
 			if (mbstrlen(option) != 2 || breadth(option) != 2) {
-				rcfile_error(N_("Two single-column characters required"));
+				jot_error(N_("Two single-column characters required"));
 				free(option);
 			} else {
 				whitespace = option;
-				whitelen[0] = parse_mbchar(whitespace, NULL, NULL);
-				whitelen[1] = parse_mbchar(whitespace + whitelen[0], NULL, NULL);
+				whitelen[0] = char_length(whitespace);
+				whitelen[1] = char_length(whitespace + whitelen[0]);
 			}
 		} else
 #endif
 #ifdef ENABLE_JUSTIFY
 		if (strcasecmp(rcopts[i].name, "punct") == 0) {
 			if (has_blank_char(option)) {
-				rcfile_error(N_("Non-blank characters required"));
+				jot_error(N_("Non-blank characters required"));
 				free(option);
 			} else
 				punct = option;
 		} else if (strcasecmp(rcopts[i].name, "brackets") == 0) {
 			if (has_blank_char(option)) {
-				rcfile_error(N_("Non-blank characters required"));
+				jot_error(N_("Non-blank characters required"));
 				free(option);
 			} else
 				brackets = option;
@@ -1177,24 +1257,18 @@ void parse_rcfile(FILE *rcstream, bool syntax_only)
 #endif
 		if (strcasecmp(rcopts[i].name, "tabsize") == 0) {
 			if (!parse_num(option, &tabsize) || tabsize <= 0) {
-				rcfile_error(N_("Requested tab size \"%s\" is invalid"),
-								option);
+				jot_error(N_("Requested tab size \"%s\" is invalid"), option);
 				tabsize = -1;
 			}
 			free(option);
 		}
 	}
 
-#ifdef ENABLE_COLOR
-	if (opensyntax && lastcolor == NULL)
-		rcfile_error(N_("Syntax \"%s\" has no color commands"),
-						live_syntax->name);
+	if (intros_only)
+		check_for_nonempty_syntax();
 
-	opensyntax = FALSE;
-#endif
-
-	free(buf);
 	fclose(rcstream);
+	free(buffer);
 	lineno = 0;
 
 	return;
@@ -1203,20 +1277,14 @@ void parse_rcfile(FILE *rcstream, bool syntax_only)
 /* Read and interpret one of the two nanorc files. */
 void parse_one_nanorc(void)
 {
-	FILE *rcstream;
-
-	/* Don't try to open directories nor devices. */
-	if (!is_good_file(nanorc))
-		return;
-
-	rcstream = fopen(nanorc, "rb");
+	FILE *rcstream = fopen(nanorc, "rb");
 
 	/* If opening the file succeeded, parse it.  Otherwise, only
 	 * complain if the file actually exists. */
 	if (rcstream != NULL)
-		parse_rcfile(rcstream, FALSE);
+		parse_rcfile(rcstream, FALSE, TRUE);
 	else if (errno != ENOENT)
-		rcfile_error(N_("Error reading %s: %s"), nanorc, strerror(errno));
+		jot_error(N_("Error reading %s: %s"), nanorc, strerror(errno));
 }
 
 bool have_nanorc(const char *path, char *name)
@@ -1235,36 +1303,25 @@ void do_rcfiles(void)
 {
 	const char *xdgconfdir;
 
-	/* First process the system-wide nanorc, if there is one. */
+	/* First process the system-wide nanorc, if it exists and is suitable. */
 	nanorc = mallocstrcpy(nanorc, SYSCONFDIR "/nanorc");
-	parse_one_nanorc();
+	if (is_good_file(nanorc))
+		parse_one_nanorc();
 
 	get_homedir();
 	xdgconfdir = getenv("XDG_CONFIG_HOME");
 
 	/* Now try the to find a nanorc file in the user's home directory or in
 	 * the XDG configuration directories, and process the first one found. */
-	if (have_nanorc(homedir, "/" HOME_RC_NAME))
-		parse_one_nanorc();
-	else if (have_nanorc(xdgconfdir, "/nano/" RCFILE_NAME))
-		parse_one_nanorc();
-	else if (have_nanorc(homedir, "/.config/nano/" RCFILE_NAME))
+	if (have_nanorc(homedir, "/" HOME_RC_NAME) ||
+				have_nanorc(xdgconfdir, "/nano/" RCFILE_NAME) ||
+				have_nanorc(homedir, "/.config/nano/" RCFILE_NAME))
 		parse_one_nanorc();
 	else if (homedir == NULL && xdgconfdir == NULL)
-		rcfile_error(N_("I can't find my home directory!  Wah!"));
+		jot_error(N_("I can't find my home directory!  Wah!"));
 
 	check_vitals_mapped();
 
-#ifdef __linux__
-	/* On a Linux console, don't start nano when there are rcfile errors,
-	 * because otherwise these error messages get wiped. */
-	if (on_a_vt && rcfile_with_errors) {
-		fprintf(stderr, _("If needed, use nano with the -I option "
-							"to adjust your nanorc settings.\n"));
-		exit(1);
-	}
-#endif
-
 	free(nanorc);
 }
 
